@@ -1,8 +1,7 @@
 use crate::error::NativeErrorExt;
 use crate::Error;
 use openh264_sys2::{
-    videoFormatI420, ISVCDecoder, ISVCDecoderVtbl, SBufferInfo, SDecodingParam, SParserBsInfo, SSysMEMBuffer, WelsCreateDecoder,
-    WelsDestroyDecoder, DECODER_OPTION, DECODER_OPTION_NUM_OF_THREADS, DECODING_STATE,
+    videoFormatI420, ISVCDecoder, ISVCDecoderVtbl, SBufferInfo, SDecodingParam, SParserBsInfo, SSysMEMBuffer, WelsCreateDecoder, WelsDestroyDecoder, DECODER_OPTION, DECODER_OPTION_ERROR_CON_IDC, DECODER_OPTION_NUM_OF_THREADS, DECODER_OPTION_TRACE_LEVEL, DECODING_STATE, WELS_LOG_DETAIL, WELS_LOG_QUIET
 };
 use std::os::raw::{c_int, c_long, c_uchar, c_void};
 use std::ptr::{addr_of_mut, null, null_mut};
@@ -38,20 +37,22 @@ impl DecoderRawAPI {
 
             WelsCreateDecoder(&mut decoder_ptr as *mut *mut *const ISVCDecoderVtbl).ok()?;
 
-            let e = Error::msg("VTable missing function.");
+            let e = || {
+                Error::msg("VTable missing function.")
+            };
 
             Ok(DecoderRawAPI {
                 decoder_ptr,
-                initialize: (*(*decoder_ptr)).Initialize.ok_or(e)?,
-                uninitialize: (*(*decoder_ptr)).Uninitialize.ok_or(e)?,
-                decode_frame: (*(*decoder_ptr)).DecodeFrame.ok_or(e)?,
-                decode_frame_no_delay: (*(*decoder_ptr)).DecodeFrameNoDelay.ok_or(e)?,
-                decode_frame2: (*(*decoder_ptr)).DecodeFrame2.ok_or(e)?,
-                flush_frame: (*(*decoder_ptr)).FlushFrame.ok_or(e)?,
-                decode_parser: (*(*decoder_ptr)).DecodeParser.ok_or(e)?,
-                decode_frame_ex: (*(*decoder_ptr)).DecodeFrameEx.ok_or(e)?,
-                set_option: (*(*decoder_ptr)).SetOption.ok_or(e)?,
-                get_option: (*(*decoder_ptr)).GetOption.ok_or(e)?,
+                initialize: (*(*decoder_ptr)).Initialize.ok_or(e())?,
+                uninitialize: (*(*decoder_ptr)).Uninitialize.ok_or(e())?,
+                decode_frame: (*(*decoder_ptr)).DecodeFrame.ok_or(e())?,
+                decode_frame_no_delay: (*(*decoder_ptr)).DecodeFrameNoDelay.ok_or(e())?,
+                decode_frame2: (*(*decoder_ptr)).DecodeFrame2.ok_or(e())?,
+                flush_frame: (*(*decoder_ptr)).FlushFrame.ok_or(e())?,
+                decode_parser: (*(*decoder_ptr)).DecodeParser.ok_or(e())?,
+                decode_frame_ex: (*(*decoder_ptr)).DecodeFrameEx.ok_or(e())?,
+                set_option: (*(*decoder_ptr)).SetOption.ok_or(e())?,
+                get_option: (*(*decoder_ptr)).GetOption.ok_or(e())?,
             })
         }
     }
@@ -113,12 +114,19 @@ impl Drop for DecoderRawAPI {
 pub struct DecoderConfig {
     params: SDecodingParam,
     num_threads: i32,
+    debug: i32,
+    error_concealment: i32,
 }
 
 impl DecoderConfig {
     /// Creates a new default encoder config.
     pub fn new() -> Self {
-        Self::default()
+        Self {
+            params: Default::default(),
+            num_threads: 0,
+            debug: 0,
+            error_concealment: 0,
+        }
     }
 
     /// Sets the number of threads.
@@ -131,6 +139,11 @@ impl DecoderConfig {
         self.num_threads = num_threads as i32;
         self
     }
+
+    pub fn debug(mut self, value: bool) -> Self {
+        self.debug = if value { WELS_LOG_DETAIL } else { WELS_LOG_QUIET };
+        self
+    }
 }
 
 /// An [OpenH264](https://github.com/cisco/openh264) decoder, converts packets to YUV.
@@ -149,10 +162,14 @@ impl Decoder {
     pub fn with_config(mut config: DecoderConfig) -> Result<Self, Error> {
         let raw = DecoderRawAPI::new()?;
 
+        // config.params.sVideoProperty.eVideoBsType = VIDEO_BITSTREAM_AVC;
+
+        #[rustfmt::skip]
         unsafe {
             raw.initialize(&config.params).ok()?;
-            raw.set_option(DECODER_OPTION_NUM_OF_THREADS, addr_of_mut!(config.num_threads).cast())
-                .ok()?;
+            raw.set_option(DECODER_OPTION_TRACE_LEVEL, addr_of_mut!(config.debug).cast()).ok()?;
+            raw.set_option(DECODER_OPTION_NUM_OF_THREADS, addr_of_mut!(config.num_threads).cast()).ok()?;
+            raw.set_option(DECODER_OPTION_ERROR_CON_IDC, addr_of_mut!(config.error_concealment).cast()).ok()?;
         };
 
         Ok(Self { raw_api: raw })
@@ -275,12 +292,19 @@ impl<'a> DecodedYUV<'a> {
     pub fn write_rgb8(&self, target: &mut [u8]) -> Result<(), Error> {
         let dim = self.dimension_rgb();
         let strides = self.strides_yuv();
+        let wanted = dim.0 * dim.1 * 3;
 
         // This needs some love, and better architecture.
         assert_eq!(self.info.iFormat, videoFormatI420 as i32);
 
-        if target.len() != (dim.0 * dim.1 * 3) as usize {
-            return Err(Error::msg("Target RGB8 array does not match image dimensions."));
+        if target.len() != wanted as usize {
+            return Err(Error::msg(&format!(
+                "Target RGB8 array does not match image dimensions. Wanted: {} * {} * 3 = {}, got {}",
+                dim.0,
+                dim.1,
+                wanted,
+                target.len()
+            )));
         }
 
         for y in 0..dim.1 {
@@ -304,4 +328,45 @@ impl<'a> DecodedYUV<'a> {
 
         Ok(())
     }
+
+    pub fn write_rgba8(&self, target: &mut [u8]) -> Result<(), Error> {
+        let dim = self.dimension_rgb();
+        let strides = self.strides_yuv();
+        let wanted = dim.0 * dim.1 * 4;
+
+        // This needs some love, and better architecture.
+        assert_eq!(self.info.iFormat, videoFormatI420 as i32);
+
+        if target.len() != wanted as usize {
+            return Err(Error::msg(&format!(
+                "Target RGB8 array does not match image dimensions. Wanted: {} * {} * 4 = {}, got {}",
+                dim.0,
+                dim.1,
+                wanted,
+                target.len()
+            )));
+        }
+
+        for y in 0..dim.1 {
+            for x in 0..dim.0 {
+                let base_tgt = (y * dim.0 + x) * 4;
+                let base_y = y * strides.0 + x;
+                let base_u = (y / 2 * strides.1) + (x / 2);
+                let base_v = (y / 2 * strides.2) + (x / 2);
+
+                let rgb_pixel = &mut target[base_tgt..base_tgt + 4];
+
+                let y = self.y[base_y] as f32;
+                let u = self.u[base_u] as f32;
+                let v = self.v[base_v] as f32;
+
+                rgb_pixel[0] = (y + 1.402 * (v - 128.0)) as u8;
+                rgb_pixel[1] = (y - 0.344 * (u - 128.0) - 0.714 * (v - 128.0)) as u8;
+                rgb_pixel[2] = (y + 1.772 * (u - 128.0)) as u8;
+                rgb_pixel[3] = 255;
+            }
+        }
+
+        Ok(())
+    }
 }
