@@ -1,13 +1,14 @@
 use openh264_sys2::{dsErrorFree, DECODING_STATE};
 use std::fmt::{Debug, Display, Formatter};
-use std::os::raw::{c_long, c_ulong};
 
 /// Error struct if something goes wrong.
-#[derive(Debug, Copy, Clone)]
+#[derive(Debug)]
 pub struct Error {
     native: i64,
     decoding_state: DECODING_STATE,
-    misc: Option<&'static str>,
+    misc: Option<String>,
+    #[cfg(feature = "backtrace")]
+    backtrace: Option<std::backtrace::Backtrace>,
 }
 
 impl Error {
@@ -16,6 +17,8 @@ impl Error {
             native,
             decoding_state: dsErrorFree,
             misc: None,
+            #[cfg(feature = "backtrace")]
+            backtrace: Some(std::backtrace::Backtrace::capture()),
         }
     }
 
@@ -25,36 +28,52 @@ impl Error {
             native: 0,
             decoding_state,
             misc: None,
+            #[cfg(feature = "backtrace")]
+            backtrace: Some(std::backtrace::Backtrace::capture()),
         }
     }
 
-    pub(crate) fn msg(msg: &'static str) -> Self {
+    pub(crate) fn msg(msg: &str) -> Self {
         Error {
             native: 0,
             decoding_state: dsErrorFree,
-            misc: Some(msg),
+            misc: Some(msg.to_string()),
+            #[cfg(feature = "backtrace")]
+            backtrace: Some(std::backtrace::Backtrace::capture()),
         }
     }
 }
 
 impl Display for Error {
     fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
-        f.write_str("OpenH264 encountered an error (TODO: improve this message): ")?;
+        f.write_str("OpenH264 encountered an error. Native:")?;
         <i64 as std::fmt::Display>::fmt(&self.native, f)?;
+        f.write_str(". Decoding State:")?;
         <std::os::raw::c_int as std::fmt::Display>::fmt(&self.decoding_state, f)?;
+        f.write_str(". User Message:")?;
         self.misc.fmt(f)?;
+
+        #[cfg(feature = "backtrace")]
+        {
+            f.write_str(". Backtraces enabled.")?;
+        }
         Ok(())
     }
 }
 
-impl std::error::Error for Error {}
+impl std::error::Error for Error {
+    #[cfg(feature = "backtrace")]
+    fn backtrace(&self) -> Option<&std::backtrace::Backtrace> {
+        self.backtrace.as_ref()
+    }
+}
 
 /// Helper trait to check the various error values produced by OpenH264.
 pub(crate) trait NativeErrorExt {
     fn ok(self) -> Result<(), Error>;
 }
 
-impl NativeErrorExt for c_ulong {
+impl NativeErrorExt for u64 {
     fn ok(self) -> Result<(), Error> {
         if self == 0 {
             Ok(())
@@ -64,7 +83,17 @@ impl NativeErrorExt for c_ulong {
     }
 }
 
-impl NativeErrorExt for c_long {
+impl NativeErrorExt for i64 {
+    fn ok(self) -> Result<(), Error> {
+        if self == 0 {
+            Ok(())
+        } else {
+            Err(Error::from_native(self as i64))
+        }
+    }
+}
+
+impl NativeErrorExt for i32 {
     fn ok(self) -> Result<(), Error> {
         if self == 0 {
             Ok(())
