@@ -1,8 +1,10 @@
+#![cfg_attr(feature = "backtrace", feature(backtrace))]
 //!
 //! [![Latest Version]][crates.io]
 //! [![docs]][docs.rs]
 //! ![BSD-2]
 //! [![Rust](https://img.shields.io/badge/rust-1.53%2B-blue.svg?maxAge=3600)](https://github.com/ralfbiedert/openh264-rust)
+//! [![Rust](https://github.com/ralfbiedert/openh264-rust/actions/workflows/rust.yml/badge.svg)](https://github.com/ralfbiedert/openh264-rust/actions/workflows/rust.yml)
 //!
 //! # OpenH264 Rust API
 //!
@@ -77,6 +79,9 @@
 //! - [ ] better enable autovectorization converting YUV to RGB,
 //! - [ ] conditionally enable assembly in `build.rs` for OpenH264.
 //!
+//! ## Compile Features
+//!
+//! - `backtrace` - Enable backtraces on errors (requires nightly)
 //!
 //! ## FAQ
 //!
@@ -121,7 +126,6 @@
 //!
 //! Especially needed:
 //!
-//! - [ ] CI testing
 //! - [ ] Encoder wrapper
 //! - [ ] Enabling of platform specific assembly (without breaking or complicating build)
 //! - [ ] Faster YUV to RGB conversion
